import * as jQuery from 'jquery';
import { Certificate, SignerService, VerifySignatureResult } from './SignerService';
import { UsernameAndPassword } from './UsernameAndPassword';
import { blobToBase64String, base64StringToBlob } from 'blob-util';

export class DocuSignSignerService extends SignerService {

    public requestVerificationCode(): Promise<boolean> {
        const usernamePassword = new UsernameAndPassword(this.authorization);
        return new Promise((resolve, reject) => {
            void jQuery.ajax({
                url: new URL('/Sign/Api/RequestOTP', this.config.serviceRoot).toString(),
                method: 'POST',
                crossDomain: true,
                dataType: 'json',
                data: {
                    Username: usernamePassword.username,
                    Password: usernamePassword.password
                },
                xhrFields: {
                    withCredentials: true
                }
            }).done((result) => {
                if (result === 1) {
                    return resolve(false);
                }
                if (result === 9) {
                    return reject(new Error('An error occurred while sending verification code'));
                }
                return resolve(true);
            }).fail((jqXHR, textStatus, errorThrown) => {
                const error = new Error(errorThrown);
                Object.assign({
                    status: jqXHR.status
                });
                return reject(error);
            });
        });
    }
    public get requiresVerificationCodeRequest(): boolean {
        return true;
    }

    private certificates: Certificate[] | null = null;

    constructor(public config: {serviceRoot: string}) {
        super();
    }

    public get status(): Promise<any> {
        throw new Error('Method not implemented.');
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    public authenticate({ username, password, rememberMe }: { username?: string; password?: string; rememberMe?: boolean; }): Promise<boolean> {
        this.certificates = [];
        return new Promise((resolve, reject) => {
            void jQuery.ajax({
                url: new URL('/Sign/Api/Certificates', this.config.serviceRoot).toString(),
                method: 'POST',
                crossDomain: true,
                dataType: 'json',
                data: {
                    'Username': username,
                    'Password': password
                },
                xhrFields: {
                    withCredentials: true
                }
            }).done((certificates: Certificate[]) => {
                this.signerAuthorization = UsernameAndPassword.create(username, password);
                this.certificates = certificates;
                return resolve(true);
            }).fail((jqXHR, textStatus, errorThrown) => {
                const error = new Error(errorThrown);
                Object.assign({
                    status: jqXHR.status
                });
                return reject(error);
            });
        });
    }
    public getCertificates(force?: boolean): Promise<Certificate[]> {
        if (force) {
            this.certificates = null;
        }
        if (Array.isArray(this.certificates)) {
            return Promise.resolve(this.certificates);
        }
        const usernamePassword = new UsernameAndPassword(this.authorization);
        return new Promise((resolve, reject) => {
            void jQuery.ajax({
                url: new URL('/Sign/Api/Certificates', this.config.serviceRoot).toString(),
                method: 'POST',
                crossDomain: true,
                dataType: 'json',
                data: {
                    Username: usernamePassword.username,
                    Password: usernamePassword.password
                },
                xhrFields: {
                    withCredentials: true
                }
            }).done((cert: Certificate) => {
                this.certificates = []
                if (cert) {
                    this.certificates.push(cert);
                }
                return resolve(this.certificates);
            }).fail((jqXHR, textStatus, errorThrown) => {
                const error = new Error(errorThrown);
                Object.assign({
                    status: jqXHR.status
                });
                return reject(error);
            });
        });
    }
    public queryStatus(): Promise<any> {
        return new Promise((resolve, reject) => {
            void jQuery.ajax({
                url: new URL('/Sign/Api/status', this.config.serviceRoot).toString(),
                method: 'GET',
                crossDomain: true,
                headers: {
                },
                dataType: 'json',
                xhrFields: {
                    withCredentials: true
                }
            }).done((data: any) => {
                return resolve(data);
            }).fail((jqXHR, textStatus, errorThrown) => {
                const error = new Error(errorThrown);
                Object.assign({
                    status: jqXHR.status
                });
                return reject(error);
            });
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    public async sign(form: FormData): Promise<any> {
        // prepare message
        const usernameAndPassword = new UsernameAndPassword(this.authorization);
        // get file
        const file = form.get('file') as File;
        const fileData: string = await blobToBase64String(file);
        // create message
        const message = {
            Username: usernameAndPassword.username,
            Password: usernameAndPassword.password,
            SignPassword: form.has('otp') ? form.get('otp').toString() : null,
            BufferToSign: [
                fileData
            ]
        };
        const result: { Success: boolean, Data: { Signature: string[] } } = await new Promise((resolve, reject) => {
            void jQuery.ajax({
                url: new URL('/Sign/Api/SignBuffer', this.config.serviceRoot).toString(),
                method: 'POST',
                crossDomain: true,
                dataType: 'json',
                data: message,
                xhrFields: {
                    withCredentials: true
                }
            }).done((data: { Success: boolean, Data: { Signature: string[] } }) => {
                return resolve(data);
            }).fail((jqXHR, textStatus, errorThrown) => {
                const error = new Error(errorThrown);
                Object.assign({
                    status: jqXHR.status
                });
                return reject(error);
            });
        });
        if (result.Success === false) {
            throw new Error('An internal error occurred while signing document');
        }
        return base64StringToBlob(result.Data.Signature[0]);
    }

    public get requiresVerificationCode(): boolean {
        return true;
    }
    public confirmVerificationCode(): Promise<any> {
        throw new Error('Method not implemented.');
    }
    public get requiresUsernamePassword(): boolean {
        return true;
    }
    public getVerificationUrl(): string {
        throw new Error('Method not implemented.');
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    public verifyDocument(file: any): Promise<VerifySignatureResult[]> {
        throw new Error('Method not implemented.');
    }

}